<?php
/**
 * PHPCompatibility, an external standard for PHP_CodeSniffer.
 *
 * @package   PHPCompatibility
 * @copyright 2012-2019 PHPCompatibility Contributors
 * @license   https://opensource.org/licenses/LGPL-3.0 LGPL3
 * @link      https://github.com/PHPCompatibility/PHPCompatibility
 */

namespace PHPCompatibility\Tests\Constants;

use PHPCompatibility\Tests\BaseSniffTest;

/**
 * Test the NewConstants sniff.
 *
 * @group newConstants
 * @group constants
 *
 * @covers \PHPCompatibility\Sniffs\Constants\NewConstantsSniff
 *
 * @since 8.1.0
 */
class NewConstantsUnitTest extends BaseSniffTest
{

    /**
     * testNewConstant
     *
     * @dataProvider dataNewConstant
     *
     * @param string $constantName      Name of the PHP constant.
     * @param string $lastVersionBefore The PHP version just *before* the constant was introduced.
     * @param array  $lines             The line numbers in the test file which apply to this constant.
     * @param string $okVersion         A PHP version in which the constant was valid.
     * @param string $testVersion       Optional. A PHP version in which to test for the error if different
     *                                  from the $lastVersionBefore.
     *
     * @return void
     */
    public function testNewConstant($constantName, $lastVersionBefore, $lines, $okVersion, $testVersion = null)
    {
        $errorVersion = (isset($testVersion)) ? $testVersion : $lastVersionBefore;
        $file         = $this->sniffFile(__FILE__, $errorVersion);
        $error        = "The constant \"{$constantName}\" is not present in PHP version {$lastVersionBefore} or earlier";
        foreach ($lines as $line) {
            $this->assertError($file, $line, $error);
        }

        $file = $this->sniffFile(__FILE__, $okVersion);
        foreach ($lines as $line) {
            $this->assertNoViolation($file, $line);
        }
    }

    /**
     * Data provider.
     *
     * @see testNewConstant()
     *
     * @return array
     */
    public function dataNewConstant()
    {
        return array(
            array('E_STRICT', '4.4', array(8), '5.0'),
            array('CURLOPT_FTP_USE_EPRT', '4.4', array(9), '5.0'),
            array('CURLOPT_NOSIGNAL', '4.4', array(10), '5.0'),
            array('CURLOPT_UNRESTRICTED_AUTH', '4.4', array(11), '5.0'),
            array('CURLOPT_BUFFERSIZE', '4.4', array(12), '5.0'),
            array('CURLOPT_HTTPAUTH', '4.4', array(13), '5.0'),
            array('CURLOPT_PROXYPORT', '4.4', array(14), '5.0'),
            array('CURLOPT_PROXYTYPE', '4.4', array(15), '5.0'),
            array('CURLOPT_SSLCERTTYPE', '4.4', array(16), '5.0'),
            array('CURLOPT_HTTP200ALIASES', '4.4', array(17), '5.0'),
            array('OPENSSL_ALGO_MD2', '4.4', array(18), '5.0'),
            array('OPENSSL_ALGO_MD4', '4.4', array(19), '5.0'),
            array('OPENSSL_ALGO_MD5', '4.4', array(20), '5.0'),
            array('OPENSSL_ALGO_SHA1', '4.4', array(21), '5.0'),
            array('OPENSSL_ALGO_DSS1', '4.4', array(22), '5.0'),
            array('T_ABSTRACT', '4.4', array(23), '5.0'),
            array('T_CATCH', '4.4', array(24), '5.0'),
            array('SORT_LOCALE_STRING', '5.0.1', array(25), '5.1', '5.0'),
            array('PHP_EOL', '5.0.1', array(26), '5.1', '5.0'),
            array('PHP_INT_MAX', '5.0.4', array(27), '5.1', '5.0'),
            array('PHP_INT_SIZE', '5.0.4', array(28), '5.1', '5.0'),
            array('__COMPILER_HALT_OFFSET__', '5.0', array(29), '5.1'),
            array('GLOB_ERR', '5.0', array(30), '5.1'),
            array('CURLOPT_AUTOREFERER', '5.0', array(31), '5.1'),
            array('CURLOPT_BINARYTRANSFER', '5.0', array(32), '5.1'),
            array('CURLOPT_COOKIESESSION', '5.0', array(33), '5.1'),
            array('CURLOPT_FTPSSLAUTH', '5.0', array(34), '5.1'),
            array('CURLOPT_PROXYAUTH', '5.0', array(35), '5.1'),
            array('CURLOPT_TIMECONDITION', '5.0', array(36), '5.1'),
            array('POSIX_F_OK', '5.0', array(37), '5.1'),
            array('POSIX_R_OK', '5.0', array(38), '5.1'),
            array('POSIX_W_OK', '5.0', array(39), '5.1'),
            array('POSIX_X_OK', '5.0', array(40), '5.1'),
            array('POSIX_S_IFBLK', '5.0', array(41), '5.1'),
            array('POSIX_S_IFCHR', '5.0', array(42), '5.1'),
            array('POSIX_S_IFIFO', '5.0', array(43), '5.1'),
            array('POSIX_S_IFREG', '5.0', array(44), '5.1'),
            array('POSIX_S_IFSOCK', '5.0', array(45), '5.1'),
            array('STREAM_IPPROTO_ICMP', '5.0', array(46), '5.1'),
            array('STREAM_IPPROTO_IP', '5.0', array(47), '5.1'),
            array('STREAM_IPPROTO_RAW', '5.0', array(48), '5.1'),
            array('STREAM_IPPROTO_TCP', '5.0', array(49), '5.1'),
            array('STREAM_IPPROTO_UDP', '5.0', array(50), '5.1'),
            array('STREAM_PF_INET', '5.0', array(51), '5.1'),
            array('STREAM_PF_INET6', '5.0', array(52), '5.1'),
            array('STREAM_PF_UNIX', '5.0', array(53), '5.1'),
            array('STREAM_SOCK_DGRAM', '5.0', array(54), '5.1'),
            array('STREAM_SOCK_RAW', '5.0', array(55), '5.1'),
            array('STREAM_SOCK_RDM', '5.0', array(56), '5.1'),
            array('STREAM_SOCK_SEQPACKET', '5.0', array(57), '5.1'),
            array('STREAM_SOCK_STREAM', '5.0', array(58), '5.1'),
            array('T_HALT_COMPILER', '5.0', array(59), '5.1'),
            array('DATE_ATOM', '5.1.0', array(60), '5.2', '5.1'),
            array('DATE_COOKIE', '5.1.0', array(61), '5.2', '5.1'),
            array('DATE_ISO8601', '5.1.0', array(62), '5.2', '5.1'),
            array('DATE_RFC822', '5.1.0', array(63), '5.2', '5.1'),
            array('DATE_RFC850', '5.1.0', array(64), '5.2', '5.1'),
            array('DATE_RFC1036', '5.1.0', array(65), '5.2', '5.1'),
            array('DATE_RFC1123', '5.1.0', array(66), '5.2', '5.1'),
            array('DATE_RFC2822', '5.1.0', array(67), '5.2', '5.1'),
            array('DATE_RFC3339', '5.1.0', array(68), '5.2', '5.1'),
            array('DATE_RSS', '5.1.0', array(69), '5.2', '5.1'),
            array('DATE_W3C', '5.1.0', array(70), '5.2', '5.1'),
            array('SUNFUNCS_RET_TIMESTAMP', '5.1.1', array(71), '5.2', '5.1'),
            array('SUNFUNCS_RET_STRING', '5.1.1', array(72), '5.2', '5.1'),
            array('SUNFUNCS_RET_DOUBLE', '5.1.1', array(73), '5.2', '5.1'),
            array('LIBXSLT_VERSION', '5.1.1', array(74), '5.2', '5.1'),
            array('LIBXSLT_DOTTED_VERSION', '5.1.1', array(75), '5.2', '5.1'),
            array('LIBEXSLT_VERSION', '5.1.1', array(76), '5.2', '5.1'),
            array('LIBEXSLT_DOTTED_VERSION', '5.1.1', array(77), '5.2', '5.1'),
            array('PHP_URL_SCHEME', '5.1.1', array(78), '5.2', '5.1'),
            array('PHP_URL_HOST', '5.1.1', array(79), '5.2', '5.1'),
            array('PHP_URL_PORT', '5.1.1', array(80), '5.2', '5.1'),
            array('PHP_URL_USER', '5.1.1', array(81), '5.2', '5.1'),
            array('PHP_URL_PASS', '5.1.1', array(82), '5.2', '5.1'),
            array('PHP_URL_PATH', '5.1.1', array(83), '5.2', '5.1'),
            array('PHP_URL_QUERY', '5.1.1', array(84), '5.2', '5.1'),
            array('PHP_URL_FRAGMENT', '5.1.1', array(85), '5.2', '5.1'),
            array('PHP_QUERY_RFC1738', '5.1.1', array(86), '5.2', '5.1'),
            array('PHP_QUERY_RFC3986', '5.1.1', array(87), '5.2', '5.1'),
            array('CURLINFO_HEADER_OUT', '5.1.2', array(88), '5.2', '5.1'),
            array('E_RECOVERABLE_ERROR', '5.1', array(89), '5.2'),
            array('M_EULER', '5.1', array(90), '5.2'),
            array('M_LNPI', '5.1', array(91), '5.2'),
            array('M_SQRT3', '5.1', array(92), '5.2'),
            array('M_SQRTPI', '5.1', array(93), '5.2'),
            array('PATHINFO_FILENAME', '5.1', array(94), '5.2'),
            array('UPLOAD_ERR_EXTENSION', '5.1', array(95), '5.2'),
            array('CURLE_FILESIZE_EXCEEDED', '5.1', array(96), '5.2'),
            array('CURLE_FTP_SSL_FAILED', '5.1', array(97), '5.2'),
            array('CURLE_LDAP_INVALID_URL', '5.1', array(98), '5.2'),
            array('CURLFTPAUTH_DEFAULT', '5.1', array(99), '5.2'),
            array('CURLFTPAUTH_SSL', '5.1', array(100), '5.2'),
            array('CURLFTPAUTH_TLS', '5.1', array(101), '5.2'),
            array('CURLFTPSSL_ALL', '5.1', array(102), '5.2'),
            array('CURLFTPSSL_CONTROL', '5.1', array(103), '5.2'),
            array('CURLFTPSSL_NONE', '5.1', array(104), '5.2'),
            array('CURLFTPSSL_TRY', '5.1', array(105), '5.2'),
            array('CURLOPT_FTP_SSL', '5.1', array(106), '5.2'),
            array('SWFTEXTFIELD_USEFONT', '5.1', array(107), '5.2'),
            array('SWFTEXTFIELD_AUTOSIZE', '5.1', array(108), '5.2'),
            array('SWF_SOUND_NOT_COMPRESSED', '5.1', array(109), '5.2'),
            array('SWF_SOUND_ADPCM_COMPRESSED', '5.1', array(110), '5.2'),
            array('SWF_SOUND_MP3_COMPRESSED', '5.1', array(111), '5.2'),
            array('SWF_SOUND_NOT_COMPRESSED_LE', '5.1', array(112), '5.2'),
            array('SWF_SOUND_NELLY_COMPRESSED', '5.1', array(113), '5.2'),
            array('SWF_SOUND_5KHZ', '5.1', array(114), '5.2'),
            array('SWF_SOUND_11KHZ', '5.1', array(115), '5.2'),
            array('SWF_SOUND_22KHZ', '5.1', array(116), '5.2'),
            array('SWF_SOUND_44KHZ', '5.1', array(117), '5.2'),
            array('SWF_SOUND_8BITS', '5.1', array(118), '5.2'),
            array('SWF_SOUND_16BITS', '5.1', array(119), '5.2'),
            array('SWF_SOUND_MONO', '5.1', array(120), '5.2'),
            array('SWF_SOUND_STEREO', '5.1', array(121), '5.2'),
            array('OPENSSL_KEYTYPE_EC', '5.1', array(122), '5.2'),
            array('OPENSSL_VERSION_NUMBER', '5.1', array(123), '5.2'),
            array('OPENSSL_VERSION_TEXT', '5.1', array(124), '5.2'),
            array('PREG_BACKTRACK_LIMIT_ERROR', '5.1', array(125), '5.2'),
            array('PREG_BAD_UTF8_ERROR', '5.1', array(126), '5.2'),
            array('PREG_INTERNAL_ERROR', '5.1', array(127), '5.2'),
            array('PREG_NO_ERROR', '5.1', array(128), '5.2'),
            array('PREG_RECURSION_LIMIT_ERROR', '5.1', array(129), '5.2'),
            array('SNMP_OID_OUTPUT_FULL', '5.1', array(130), '5.2'),
            array('SNMP_OID_OUTPUT_NUMERIC', '5.1', array(131), '5.2'),
            array('MSG_EAGAIN', '5.1', array(132), '5.2'),
            array('MSG_ENOMSG', '5.1', array(133), '5.2'),
            array('CURLOPT_TCP_NODELAY', '5.2.0', array(134), '5.3', '5.2'),
            array('STREAM_SHUT_RD', '5.2.0', array(135), '5.3', '5.2'),
            array('STREAM_SHUT_WR', '5.2.0', array(136), '5.3', '5.2'),
            array('STREAM_SHUT_RDWR', '5.2.0', array(137), '5.3', '5.2'),
            array('GMP_VERSION', '5.2.1', array(138), '5.3', '5.2'),
            array('CURLOPT_TIMEOUT_MS', '5.2.2', array(139), '5.3', '5.2'),
            array('CURLOPT_CONNECTTIMEOUT_MS', '5.2.2', array(140), '5.3', '5.2'),
            array('CURLOPT_PRIVATE', '5.2.3', array(141), '5.3', '5.2'),
            array('CURLINFO_PRIVATE', '5.2.3', array(142), '5.3', '5.2'),
            array('GD_VERSION', '5.2.3', array(143), '5.3', '5.2'),
            array('GD_MAJOR_VERSION', '5.2.3', array(144), '5.3', '5.2'),
            array('GD_MINOR_VERSION', '5.2.3', array(145), '5.3', '5.2'),
            array('GD_RELEASE_VERSION', '5.2.3', array(146), '5.3', '5.2'),
            array('GD_EXTRA_VERSION', '5.2.3', array(147), '5.3', '5.2'),
            array('PCRE_VERSION', '5.2.3', array(148), '5.3', '5.2'),
            array('PHP_MAJOR_VERSION', '5.2.6', array(149), '5.3', '5.2'),
            array('PHP_MINOR_VERSION', '5.2.6', array(150), '5.3', '5.2'),
            array('PHP_RELEASE_VERSION', '5.2.6', array(151), '5.3', '5.2'),
            array('PHP_VERSION_ID', '5.2.6', array(152), '5.3', '5.2'),
            array('PHP_EXTRA_VERSION', '5.2.6', array(153), '5.3', '5.2'),
            array('PHP_ZTS', '5.2.6', array(154), '5.3', '5.2'),
            array('PHP_DEBUG', '5.2.6', array(155), '5.3', '5.2'),
            array('FILE_BINARY', '5.2.6', array(156), '5.3', '5.2'),
            array('FILE_TEXT', '5.2.6', array(157), '5.3', '5.2'),
            array('TCP_NODELAY', '5.2.6', array(158), '5.3', '5.2'),
            array('CURLOPT_PROTOCOLS', '5.2.9', array(159), '5.3', '5.2'),
            array('CURLOPT_REDIR_PROTOCOLS', '5.2.9', array(160), '5.3', '5.2'),
            array('CURLPROXY_SOCKS4', '5.2.9', array(161), '5.3', '5.2'),
            array('LIBXML_PARSEHUGE', '5.2.11', array(162), '5.3', '5.2'),
            array('ENT_IGNORE', '5.2', array(163), '5.3'),
            array('E_DEPRECATED', '5.2', array(164), '5.3'),
            array('E_USER_DEPRECATED', '5.2', array(165), '5.3'),
            array('INI_SCANNER_NORMAL', '5.2', array(166), '5.3'),
            array('INI_SCANNER_RAW', '5.2', array(167), '5.3'),
            array('PHP_MAXPATHLEN', '5.2', array(168), '5.3'),
            array('PHP_WINDOWS_NT_DOMAIN_CONTROLLER', '5.2', array(169), '5.3'),
            array('PHP_WINDOWS_NT_SERVER', '5.2', array(170), '5.3'),
            array('PHP_WINDOWS_NT_WORKSTATION', '5.2', array(171), '5.3'),
            array('PHP_WINDOWS_VERSION_BUILD', '5.2', array(172), '5.3'),
            array('PHP_WINDOWS_VERSION_MAJOR', '5.2', array(173), '5.3'),
            array('PHP_WINDOWS_VERSION_MINOR', '5.2', array(174), '5.3'),
            array('PHP_WINDOWS_VERSION_PLATFORM', '5.2', array(175), '5.3'),
            array('PHP_WINDOWS_VERSION_PRODUCTTYPE', '5.2', array(176), '5.3'),
            array('PHP_WINDOWS_VERSION_SP_MAJOR', '5.2', array(177), '5.3'),
            array('PHP_WINDOWS_VERSION_SP_MINOR', '5.2', array(178), '5.3'),
            array('PHP_WINDOWS_VERSION_SUITEMASK', '5.2', array(179), '5.3'),
            array('CURLINFO_CERTINFO', '5.2', array(180), '5.3'),
            array('CURLOPT_PROGRESSFUNCTION', '5.2', array(181), '5.3'),
            array('CURLE_SSH', '5.2', array(182), '5.3'),
            array('IMG_FILTER_PIXELATE', '5.2', array(183), '5.3'),
            array('IMAGETYPE_ICO', '5.2', array(184), '5.3'),
            array('FILEINFO_MIME_TYPE', '5.2', array(185), '5.3'),
            array('FILEINFO_MIME_ENCODING', '5.2', array(186), '5.3'),
            array('JSON_ERROR_CTRL_CHAR', '5.2', array(187), '5.3'),
            array('JSON_ERROR_DEPTH', '5.2', array(188), '5.3'),
            array('JSON_ERROR_NONE', '5.2', array(189), '5.3'),
            array('JSON_ERROR_STATE_MISMATCH', '5.2', array(190), '5.3'),
            array('JSON_ERROR_SYNTAX', '5.2', array(191), '5.3'),
            array('JSON_FORCE_OBJECT', '5.2', array(192), '5.3'),
            array('JSON_HEX_TAG', '5.2', array(193), '5.3'),
            array('JSON_HEX_AMP', '5.2', array(194), '5.3'),
            array('JSON_HEX_APOS', '5.2', array(195), '5.3'),
            array('JSON_HEX_QUOT', '5.2', array(196), '5.3'),
            array('LDAP_OPT_NETWORK_TIMEOUT', '5.2', array(197), '5.3'),
            array('LIBXML_LOADED_VERSION', '5.2', array(198), '5.3'),
            array('PHP_ROUND_HALF_UP', '5.2', array(199), '5.3'),
            array('PHP_ROUND_HALF_DOWN', '5.2', array(200), '5.3'),
            array('PHP_ROUND_HALF_EVEN', '5.2', array(201), '5.3'),
            array('PHP_ROUND_HALF_ODD', '5.2', array(202), '5.3'),
            array('MYSQLI_OPT_INT_AND_FLOAT_NATIVE', '5.2', array(203), '5.3'),
            array('MYSQLI_OPT_NET_CMD_BUFFER_SIZE', '5.2', array(204), '5.3'),
            array('MYSQLI_OPT_NET_READ_BUFFER_SIZE', '5.2', array(205), '5.3'),
            array('MYSQLI_OPT_SSL_VERIFY_SERVER_CERT', '5.2', array(206), '5.3'),
            array('OCI_CRED_EXT', '5.2', array(207), '5.3'),
            array('PREG_BAD_UTF8_OFFSET_ERROR', '5.2', array(208), '5.3'),
            array('BUS_ADRALN', '5.2', array(209), '5.3'),
            array('BUS_ADRERR', '5.2', array(210), '5.3'),
            array('BUS_OBJERR', '5.2', array(211), '5.3'),
            array('CLD_CONTIUNED', '5.2', array(212), '5.3'),
            array('CLD_DUMPED', '5.2', array(213), '5.3'),
            array('CLD_EXITED', '5.2', array(214), '5.3'),
            array('CLD_KILLED', '5.2', array(215), '5.3'),
            array('CLD_STOPPED', '5.2', array(216), '5.3'),
            array('CLD_TRAPPED', '5.2', array(217), '5.3'),
            array('FPE_FLTDIV', '5.2', array(218), '5.3'),
            array('FPE_FLTINV', '5.2', array(219), '5.3'),
            array('FPE_FLTOVF', '5.2', array(220), '5.3'),
            array('FPE_FLTRES', '5.2', array(221), '5.3'),
            array('FPE_FLTSUB', '5.2', array(222), '5.3'),
            array('FPE_FLTUND', '5.2', array(223), '5.3'),
            array('FPE_INTDIV', '5.2', array(224), '5.3'),
            array('FPE_INTOVF', '5.2', array(225), '5.3'),
            array('ILL_BADSTK', '5.2', array(226), '5.3'),
            array('ILL_COPROC', '5.2', array(227), '5.3'),
            array('ILL_ILLADR', '5.2', array(228), '5.3'),
            array('ILL_ILLOPC', '5.2', array(229), '5.3'),
            array('ILL_ILLOPN', '5.2', array(230), '5.3'),
            array('ILL_ILLTRP', '5.2', array(231), '5.3'),
            array('ILL_PRVOPC', '5.2', array(232), '5.3'),
            array('ILL_PRVREG', '5.2', array(233), '5.3'),
            array('POLL_ERR', '5.2', array(234), '5.3'),
            array('POLL_HUP', '5.2', array(235), '5.3'),
            array('POLL_IN', '5.2', array(236), '5.3'),
            array('POLL_MSG', '5.2', array(237), '5.3'),
            array('POLL_OUT', '5.2', array(238), '5.3'),
            array('POLL_PRI', '5.2', array(239), '5.3'),
            array('SEGV_ACCERR', '5.2', array(240), '5.3'),
            array('SEGV_MAPERR', '5.2', array(241), '5.3'),
            array('SI_ASYNCIO', '5.2', array(242), '5.3'),
            array('SI_KERNEL', '5.2', array(243), '5.3'),
            array('SI_MSGGQ', '5.2', array(244), '5.3'),
            array('SI_NOINFO', '5.2', array(245), '5.3'),
            array('SI_QUEUE', '5.2', array(246), '5.3'),
            array('SI_SIGIO', '5.2', array(247), '5.3'),
            array('SI_TIMER', '5.2', array(248), '5.3'),
            array('SI_TKILL', '5.2', array(249), '5.3'),
            array('SI_USER', '5.2', array(250), '5.3'),
            array('SIG_BLOCK', '5.2', array(251), '5.3'),
            array('SIG_SETMASK', '5.2', array(252), '5.3'),
            array('SIG_UNBLOCK', '5.2', array(253), '5.3'),
            array('TRAP_BRKPT', '5.2', array(254), '5.3'),
            array('TRAP_TRACE', '5.2', array(255), '5.3'),
            array('T_DIR', '5.2', array(256), '5.3'),
            array('T_GOTO', '5.2', array(257), '5.3'),
            array('T_NAMESPACE', '5.2', array(258), '5.3'),
            array('T_NS_C', '5.2', array(259), '5.3'),
            array('T_NS_SEPARATOR', '5.2', array(260), '5.3'),
            array('T_USE', '5.2', array(261), '5.3'),
            array('OCI_NO_AUTO_COMMIT', '5.3.1', array(262), '5.4', '5.3'),
            array('OPENSSL_TLSEXT_SERVER_NAME', '5.3.1', array(263), '5.4', '5.3'),
            array('JSON_ERROR_UTF8', '5.3.2', array(264), '5.4', '5.3'),
            array('JSON_NUMERIC_CHECK', '5.3.2', array(265), '5.4', '5.3'),
            array('DEBUG_BACKTRACE_IGNORE_ARGS', '5.3.5', array(266), '5.4', '5.3'),
            array('CURLINFO_REDIRECT_URL', '5.3.6', array(267), '5.4', '5.3'),
            array('PHP_MANDIR', '5.3.6', array(268), '5.4', '5.3'),
            array('PHP_BINARY', '5.3', array(269), '5.4'),
            array('SORT_NATURAL', '5.3', array(270), '5.4'),
            array('SORT_FLAG_CASE', '5.3', array(271), '5.4'),
            array('ENT_HTML401', '5.3', array(272), '5.4'),
            array('ENT_XML1', '5.3', array(273), '5.4'),
            array('ENT_XHTML', '5.3', array(274), '5.4'),
            array('ENT_HTML5', '5.3', array(275), '5.4'),
            array('ENT_SUBSTITUTE', '5.3', array(276), '5.4'),
            array('ENT_DISALLOWED', '5.3', array(277), '5.4'),
            array('IPPROTO_IP', '5.3', array(278), '5.4'),
            array('IPPROTO_IPV6', '5.3', array(279), '5.4'),
            array('IPV6_MULTICAST_HOPS', '5.3', array(280), '5.4'),
            array('IPV6_MULTICAST_IF', '5.3', array(281), '5.4'),
            array('IPV6_MULTICAST_LOOP', '5.3', array(282), '5.4'),
            array('IP_MULTICAST_IF', '5.3', array(283), '5.4'),
            array('IP_MULTICAST_LOOP', '5.3', array(284), '5.4'),
            array('IP_MULTICAST_TTL', '5.3', array(285), '5.4'),
            array('MCAST_JOIN_GROUP', '5.3', array(286), '5.4'),
            array('MCAST_LEAVE_GROUP', '5.3', array(287), '5.4'),
            array('MCAST_BLOCK_SOURCE', '5.3', array(288), '5.4'),
            array('MCAST_UNBLOCK_SOURCE', '5.3', array(289), '5.4'),
            array('MCAST_JOIN_SOURCE_GROUP', '5.3', array(290), '5.4'),
            array('MCAST_LEAVE_SOURCE_GROUP', '5.3', array(291), '5.4'),
            array('CURLOPT_MAX_RECV_SPEED_LARGE', '5.3', array(292), '5.4'),
            array('CURLOPT_MAX_SEND_SPEED_LARGE', '5.3', array(293), '5.4'),
            array('SCANDIR_SORT_ASCENDING', '5.3', array(294), '5.4'),
            array('SCANDIR_SORT_DESCENDING', '5.3', array(295), '5.4'),
            array('SCANDIR_SORT_NONE', '5.3', array(296), '5.4'),
            array('LIBXML_HTML_NODEFDTD', '5.3', array(297), '5.4'),
            array('LIBXML_HTML_NOIMPLIED', '5.3', array(298), '5.4'),
            array('LIBXML_PEDANTIC', '5.3', array(299), '5.4'),
            array('OPENSSL_CIPHER_AES_128_CBC', '5.3', array(300), '5.4'),
            array('OPENSSL_CIPHER_AES_192_CBC', '5.3', array(301), '5.4'),
            array('OPENSSL_CIPHER_AES_256_CBC', '5.3', array(302), '5.4'),
            array('OPENSSL_RAW_DATA', '5.3', array(303), '5.4'),
            array('OPENSSL_ZERO_PADDING', '5.3', array(304), '5.4'),
            array('PHP_OUTPUT_HANDLER_CLEAN', '5.3', array(305), '5.4'),
            array('PHP_OUTPUT_HANDLER_CLEANABLE', '5.3', array(306), '5.4'),
            array('PHP_OUTPUT_HANDLER_DISABLED', '5.3', array(307), '5.4'),
            array('PHP_OUTPUT_HANDLER_FINAL', '5.3', array(308), '5.4'),
            array('PHP_OUTPUT_HANDLER_FLUSH', '5.3', array(309), '5.4'),
            array('PHP_OUTPUT_HANDLER_FLUSHABLE', '5.3', array(310), '5.4'),
            array('PHP_OUTPUT_HANDLER_REMOVABLE', '5.3', array(311), '5.4'),
            array('PHP_OUTPUT_HANDLER_STARTED', '5.3', array(312), '5.4'),
            array('PHP_OUTPUT_HANDLER_STDFLAGS', '5.3', array(313), '5.4'),
            array('PHP_OUTPUT_HANDLER_WRITE', '5.3', array(314), '5.4'),
            array('PHP_SESSION_ACTIVE', '5.3', array(315), '5.4'),
            array('PHP_SESSION_DISABLED', '5.3', array(316), '5.4'),
            array('PHP_SESSION_NONE', '5.3', array(317), '5.4'),
            array('STREAM_META_ACCESS', '5.3', array(318), '5.4'),
            array('STREAM_META_GROUP', '5.3', array(319), '5.4'),
            array('STREAM_META_GROUP_NAME', '5.3', array(320), '5.4'),
            array('STREAM_META_OWNER', '5.3', array(321), '5.4'),
            array('STREAM_META_OWNER_NAME', '5.3', array(322), '5.4'),
            array('STREAM_META_TOUCH', '5.3', array(323), '5.4'),
            array('U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR', '5.3', array(324), '5.4'),
            array('IDNA_CHECK_BIDI', '5.3', array(325), '5.4'),
            array('IDNA_CHECK_CONTEXTJ', '5.3', array(326), '5.4'),
            array('IDNA_NONTRANSITIONAL_TO_ASCII', '5.3', array(327), '5.4'),
            array('IDNA_NONTRANSITIONAL_TO_UNICODE', '5.3', array(328), '5.4'),
            array('INTL_IDNA_VARIANT_2003', '5.3', array(329), '5.4'),
            array('INTL_IDNA_VARIANT_UTS46', '5.3', array(330), '5.4'),
            array('IDNA_ERROR_EMPTY_LABEL', '5.3', array(331), '5.4'),
            array('IDNA_ERROR_LABEL_TOO_LONG', '5.3', array(332), '5.4'),
            array('IDNA_ERROR_DOMAIN_NAME_TOO_LONG', '5.3', array(333), '5.4'),
            array('IDNA_ERROR_LEADING_HYPHEN', '5.3', array(334), '5.4'),
            array('IDNA_ERROR_TRAILING_HYPHEN', '5.3', array(335), '5.4'),
            array('IDNA_ERROR_HYPHEN_3_4', '5.3', array(336), '5.4'),
            array('IDNA_ERROR_LEADING_COMBINING_MARK', '5.3', array(337), '5.4'),
            array('IDNA_ERROR_DISALLOWED', '5.3', array(338), '5.4'),
            array('IDNA_ERROR_PUNYCODE', '5.3', array(339), '5.4'),
            array('IDNA_ERROR_LABEL_HAS_DOT', '5.3', array(340), '5.4'),
            array('IDNA_ERROR_INVALID_ACE_LABEL', '5.3', array(341), '5.4'),
            array('IDNA_ERROR_BIDI', '5.3', array(342), '5.4'),
            array('IDNA_ERROR_CONTEXTJ', '5.3', array(343), '5.4'),
            array('JSON_PRETTY_PRINT', '5.3', array(344), '5.4'),
            array('JSON_UNESCAPED_SLASHES', '5.3', array(345), '5.4'),
            array('JSON_UNESCAPED_UNICODE', '5.3', array(346), '5.4'),
            array('JSON_BIGINT_AS_STRING', '5.3', array(347), '5.4'),
            array('JSON_OBJECT_AS_ARRAY', '5.3', array(348), '5.4'),
            array('SNMP_OID_OUTPUT_SUFFIX', '5.3', array(349), '5.4'),
            array('SNMP_OID_OUTPUT_MODULE', '5.3', array(350), '5.4'),
            array('SNMP_OID_OUTPUT_UCD', '5.3', array(351), '5.4'),
            array('SNMP_OID_OUTPUT_NONE', '5.3', array(352), '5.4'),
            array('T_INSTEADOF', '5.3', array(353), '5.4'),
            array('T_TRAIT', '5.3', array(354), '5.4'),
            array('T_TRAIT_C', '5.3', array(355), '5.4'),
            array('CURLINFO_PRIMARY_IP', '5.4.6', array(356), '5.5', '5.4'),
            array('CURLINFO_PRIMARY_PORT', '5.4.6', array(357), '5.5', '5.4'),
            array('CURLINFO_LOCAL_IP', '5.4.6', array(358), '5.5', '5.4'),
            array('CURLINFO_LOCAL_PORT', '5.4.6', array(359), '5.5', '5.4'),
            array('OPENSSL_ALGO_RMD160', '5.4.7', array(360), '5.5', '5.4'),
            array('OPENSSL_ALGO_SHA224', '5.4.7', array(361), '5.5', '5.4'),
            array('OPENSSL_ALGO_SHA256', '5.4.7', array(362), '5.5', '5.4'),
            array('OPENSSL_ALGO_SHA384', '5.4.7', array(363), '5.5', '5.4'),
            array('OPENSSL_ALGO_SHA512', '5.4.7', array(364), '5.5', '5.4'),
            array('FILTER_VALIDATE_MAC', '5.4', array(365), '5.5'),
            array('IMG_AFFINE_TRANSLATE', '5.4', array(366), '5.5'),
            array('IMG_AFFINE_SCALE', '5.4', array(367), '5.5'),
            array('IMG_AFFINE_ROTATE', '5.4', array(368), '5.5'),
            array('IMG_AFFINE_SHEAR_HORIZONTAL', '5.4', array(369), '5.5'),
            array('IMG_AFFINE_SHEAR_VERTICAL', '5.4', array(370), '5.5'),
            array('IMG_CROP_DEFAULT', '5.4', array(371), '5.5'),
            array('IMG_CROP_TRANSPARENT', '5.4', array(372), '5.5'),
            array('IMG_CROP_BLACK', '5.4', array(373), '5.5'),
            array('IMG_CROP_WHITE', '5.4', array(374), '5.5'),
            array('IMG_CROP_SIDES', '5.4', array(375), '5.5'),
            array('IMG_FLIP_BOTH', '5.4', array(376), '5.5'),
            array('IMG_FLIP_HORIZONTAL', '5.4', array(377), '5.5'),
            array('IMG_FLIP_VERTICAL', '5.4', array(378), '5.5'),
            array('IMG_BELL', '5.4', array(379), '5.5'),
            array('IMG_BESSEL', '5.4', array(380), '5.5'),
            array('IMG_BILINEAR_FIXED', '5.4', array(381), '5.5'),
            array('IMG_BICUBIC', '5.4', array(382), '5.5'),
            array('IMG_BICUBIC_FIXED', '5.4', array(383), '5.5'),
            array('IMG_BLACKMAN', '5.4', array(384), '5.5'),
            array('IMG_BOX', '5.4', array(385), '5.5'),
            array('IMG_BSPLINE', '5.4', array(386), '5.5'),
            array('IMG_CATMULLROM', '5.4', array(387), '5.5'),
            array('IMG_GAUSSIAN', '5.4', array(388), '5.5'),
            array('IMG_GENERALIZED_CUBIC', '5.4', array(389), '5.5'),
            array('IMG_HERMITE', '5.4', array(390), '5.5'),
            array('IMG_HAMMING', '5.4', array(391), '5.5'),
            array('IMG_HANNING', '5.4', array(392), '5.5'),
            array('IMG_MITCHELL', '5.4', array(393), '5.5'),
            array('IMG_POWER', '5.4', array(394), '5.5'),
            array('IMG_QUADRATIC', '5.4', array(395), '5.5'),
            array('IMG_SINC', '5.4', array(396), '5.5'),
            array('IMG_NEAREST_NEIGHBOUR', '5.4', array(397), '5.5'),
            array('IMG_WEIGHTED4', '5.4', array(398), '5.5'),
            array('IMG_TRIANGLE', '5.4', array(399), '5.5'),
            array('JSON_ERROR_RECURSION', '5.4', array(400), '5.5'),
            array('JSON_ERROR_INF_OR_NAN', '5.4', array(401), '5.5'),
            array('JSON_ERROR_UNSUPPORTED_TYPE', '5.4', array(402), '5.5'),
            array('JSON_PARTIAL_OUTPUT_ON_ERROR', '5.4', array(403), '5.5'),
            array('MYSQLI_SERVER_PUBLIC_KEY', '5.4', array(404), '5.5'),
            array('CURLOPT_SHARE', '5.4', array(405), '5.5'),
            array('CURLOPT_SSL_OPTIONS', '5.4', array(406), '5.5'),
            array('CURLSSLOPT_ALLOW_BEAST', '5.4', array(407), '5.5'),
            array('CURLOPT_USERNAME', '5.4', array(408), '5.5'),
            array('CURLINFO_RESPONSE_CODE', '5.4', array(409), '5.5'),
            array('CURLINFO_HTTP_CONNECTCODE', '5.4', array(410), '5.5'),
            array('CURLINFO_HTTPAUTH_AVAIL', '5.4', array(411), '5.5'),
            array('CURLINFO_PROXYAUTH_AVAIL', '5.4', array(412), '5.5'),
            array('CURLINFO_OS_ERRNO', '5.4', array(413), '5.5'),
            array('CURLINFO_NUM_CONNECTS', '5.4', array(414), '5.5'),
            array('CURLINFO_SSL_ENGINES', '5.4', array(415), '5.5'),
            array('CURLINFO_COOKIELIST', '5.4', array(416), '5.5'),
            array('CURLINFO_FTP_ENTRY_PATH', '5.4', array(417), '5.5'),
            array('CURLINFO_APPCONNECT_TIME', '5.4', array(418), '5.5'),
            array('CURLINFO_CONDITION_UNMET', '5.4', array(419), '5.5'),
            array('CURLINFO_RTSP_CLIENT_CSEQ', '5.4', array(420), '5.5'),
            array('CURLINFO_RTSP_CSEQ_RECV', '5.4', array(421), '5.5'),
            array('CURLINFO_RTSP_SERVER_CSEQ', '5.4', array(422), '5.5'),
            array('CURLINFO_RTSP_SESSION_ID', '5.4', array(423), '5.5'),
            array('CURLMOPT_PIPELINING', '5.4', array(424), '5.5'),
            array('CURLMOPT_MAXCONNECTS', '5.4', array(425), '5.5'),
            array('CURLPAUSE_ALL', '5.4', array(426), '5.5'),
            array('CURLPAUSE_CONT', '5.4', array(427), '5.5'),
            array('CURLPAUSE_RECV', '5.4', array(428), '5.5'),
            array('CURLPAUSE_RECV_CONT', '5.4', array(429), '5.5'),
            array('CURLPAUSE_SEND', '5.4', array(430), '5.5'),
            array('CURLPAUSE_SEND_CONT', '5.4', array(431), '5.5'),
            array('SOAP_SSL_METHOD_TLS', '5.4', array(432), '5.5'),
            array('SOAP_SSL_METHOD_SSLv2', '5.4', array(433), '5.5'),
            array('SOAP_SSL_METHOD_SSLv3', '5.4', array(434), '5.5'),
            array('SOAP_SSL_METHOD_SSLv23', '5.4', array(435), '5.5'),
            array('T_FINALLY', '5.4', array(436), '5.5'),
            array('T_YIELD', '5.4', array(437), '5.5'),
            array('PASSWORD_BCRYPT', '5.4', array(696), '5.5'),
            array('PASSWORD_DEFAULT', '5.4', array(697), '5.5'),
            array('PASSWORD_BCRYPT_DEFAULT_COST', '5.4', array(698), '5.5'),
            array('LIBXML_SCHEMA_CREATE', '5.5.1', array(438), '5.6', '5.5'),
            array('CURL_SSLVERSION_TLSv1_0', '5.5.18', array(439), '5.6', '5.5'),
            array('CURL_SSLVERSION_TLSv1_1', '5.5.18', array(440), '5.6', '5.5'),
            array('CURL_SSLVERSION_TLSv1_2', '5.5.18', array(441), '5.6', '5.5'),
            array('CURLPROXY_SOCKS4A', '5.5.22', array(442), '5.6', '5.5'),
            array('CURLPROXY_SOCKS5_HOSTNAME', '5.5.22', array(443), '5.6', '5.5'),
            array('CURL_VERSION_HTTP2', '5.5.23', array(444), '5.6', '5.5'),
            array('ARRAY_FILTER_USE_KEY', '5.5', array(445), '5.6'),
            array('ARRAY_FILTER_USE_BOTH', '5.5', array(446), '5.6'),
            array('LDAP_ESCAPE_DN', '5.5', array(447), '5.6'),
            array('LDAP_ESCAPE_FILTER', '5.5', array(448), '5.6'),
            array('OPENSSL_DEFAULT_STREAM_CIPHERS', '5.5', array(449), '5.6'),
            array('STREAM_CRYPTO_METHOD_ANY_CLIENT', '5.5', array(450), '5.6'),
            array('STREAM_CRYPTO_METHOD_ANY_SERVER', '5.5', array(451), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_0_CLIENT', '5.5', array(452), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_0_SERVER', '5.5', array(453), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_1_CLIENT', '5.5', array(454), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_1_SERVER', '5.5', array(455), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT', '5.5', array(456), '5.6'),
            array('STREAM_CRYPTO_METHOD_TLSv1_2_SERVER', '5.5', array(457), '5.6'),
            array('PGSQL_CONNECT_ASYNC', '5.5', array(458), '5.6'),
            array('PGSQL_CONNECTION_AUTH_OK', '5.5', array(459), '5.6'),
            array('PGSQL_CONNECTION_AWAITING_RESPONSE', '5.5', array(460), '5.6'),
            array('PGSQL_CONNECTION_MADE', '5.5', array(461), '5.6'),
            array('PGSQL_CONNECTION_SETENV', '5.5', array(462), '5.6'),
            array('PGSQL_CONNECTION_SSL_STARTUP', '5.5', array(463), '5.6'),
            array('PGSQL_CONNECTION_STARTED', '5.5', array(464), '5.6'),
            array('PGSQL_DML_ESCAPE', '5.5', array(465), '5.6'),
            array('PGSQL_POLLING_ACTIVE', '5.5', array(466), '5.6'),
            array('PGSQL_POLLING_FAILED', '5.5', array(467), '5.6'),
            array('PGSQL_POLLING_OK', '5.5', array(468), '5.6'),
            array('PGSQL_POLLING_READING', '5.5', array(469), '5.6'),
            array('PGSQL_POLLING_WRITING', '5.5', array(470), '5.6'),
            array('T_ELLIPSIS', '5.5', array(471), '5.6'),
            array('T_POW', '5.5', array(472), '5.6'),
            array('T_POW_EQUAL', '5.5', array(473), '5.6'),
            array('INI_SCANNER_TYPED', '5.6.0', array(474), '7.0', '5.6'),
            array('JSON_PRESERVE_ZERO_FRACTION', '5.6.5', array(475), '7.0', '5.6'),
            array('MYSQLI_CLIENT_SSL_DONT_VERIFY_SERVER_CERT', '5.6.15', array(476), '7.0', '5.6'),
            array('IMG_WEBP', '5.6.24', array(477), '7.0', '5.6'),
            array('TOKEN_PARSE', '5.6', array(478), '7.0'),
            array('FILTER_VALIDATE_DOMAIN', '5.6', array(479), '7.0'),
            array('PHP_INT_MIN', '5.6', array(480), '7.0'),
            array('CURLPIPE_NOTHING', '5.6', array(481), '7.0'),
            array('CURLPIPE_HTTP1', '5.6', array(482), '7.0'),
            array('CURLPIPE_MULTIPLEX', '5.6', array(483), '7.0'),
            array('JSON_ERROR_INVALID_PROPERTY_NAME', '5.6', array(484), '7.0'),
            array('JSON_ERROR_UTF16', '5.6', array(485), '7.0'),
            array('LIBXML_BIGLINES', '5.6', array(486), '7.0'),
            array('PREG_JIT_STACKLIMIT_ERROR', '5.6', array(487), '7.0'),
            array('POSIX_RLIMIT_AS', '5.6', array(488), '7.0'),
            array('POSIX_RLIMIT_CORE', '5.6', array(489), '7.0'),
            array('POSIX_RLIMIT_CPU', '5.6', array(490), '7.0'),
            array('POSIX_RLIMIT_DATA', '5.6', array(491), '7.0'),
            array('POSIX_RLIMIT_FSIZE', '5.6', array(492), '7.0'),
            array('POSIX_RLIMIT_LOCKS', '5.6', array(493), '7.0'),
            array('POSIX_RLIMIT_MEMLOCK', '5.6', array(494), '7.0'),
            array('POSIX_RLIMIT_MSGQUEUE', '5.6', array(495), '7.0'),
            array('POSIX_RLIMIT_NICE', '5.6', array(496), '7.0'),
            array('POSIX_RLIMIT_NOFILE', '5.6', array(497), '7.0'),
            array('POSIX_RLIMIT_NPROC', '5.6', array(498), '7.0'),
            array('POSIX_RLIMIT_RSS', '5.6', array(499), '7.0'),
            array('POSIX_RLIMIT_RTPRIO', '5.6', array(500), '7.0'),
            array('POSIX_RLIMIT_RTTIME', '5.6', array(501), '7.0'),
            array('POSIX_RLIMIT_SIGPENDING', '5.6', array(502), '7.0'),
            array('POSIX_RLIMIT_STACK', '5.6', array(503), '7.0'),
            array('POSIX_RLIMIT_INFINITY', '5.6', array(504), '7.0'),
            array('T_COALESCE', '5.6', array(822), '7.0'),
            array('T_SPACESHIP', '5.6', array(505), '7.0'),
            array('T_YIELD_FROM', '5.6', array(823), '7.0'),
            array('ZLIB_ENCODING_RAW', '5.6', array(506), '7.0'),
            array('ZLIB_ENCODING_DEFLATE', '5.6', array(507), '7.0'),
            array('ZLIB_ENCODING_GZIP', '5.6', array(508), '7.0'),
            array('ZLIB_FILTERED', '5.6', array(509), '7.0'),
            array('ZLIB_HUFFMAN_ONLY', '5.6', array(510), '7.0'),
            array('ZLIB_FIXED', '5.6', array(511), '7.0'),
            array('ZLIB_RLE', '5.6', array(512), '7.0'),
            array('ZLIB_DEFAULT_STRATEGY', '5.6', array(513), '7.0'),
            array('ZLIB_BLOCK', '5.6', array(514), '7.0'),
            array('ZLIB_FINISH', '5.6', array(515), '7.0'),
            array('ZLIB_FULL_FLUSH', '5.6', array(516), '7.0'),
            array('ZLIB_NO_FLUSH', '5.6', array(517), '7.0'),
            array('ZLIB_PARTIAL_FLUSH', '5.6', array(518), '7.0'),
            array('ZLIB_SYNC_FLUSH', '5.6', array(519), '7.0'),
            array('CURL_HTTP_VERSION_2', '7.0.6', array(520), '7.1', '7.0'),
            array('CURL_HTTP_VERSION_2_PRIOR_KNOWLEDGE', '7.0.6', array(521), '7.1', '7.0'),
            array('CURL_HTTP_VERSION_2TLS', '7.0.6', array(522), '7.1', '7.0'),
            array('CURL_REDIR_POST_301', '7.0.6', array(523), '7.1', '7.0'),
            array('CURL_REDIR_POST_302', '7.0.6', array(524), '7.1', '7.0'),
            array('CURL_REDIR_POST_303', '7.0.6', array(525), '7.1', '7.0'),
            array('CURL_REDIR_POST_ALL', '7.0.6', array(526), '7.1', '7.0'),
            array('CURL_VERSION_KERBEROS5', '7.0.6', array(527), '7.1', '7.0'),
            array('CURL_VERSION_PSL', '7.0.6', array(528), '7.1', '7.0'),
            array('CURL_VERSION_UNIX_SOCKETS', '7.0.6', array(529), '7.1', '7.0'),
            array('CURLAUTH_NEGOTIATE', '7.0.6', array(530), '7.1', '7.0'),
            array('CURLAUTH_NTLM_WB', '7.0.6', array(531), '7.1', '7.0'),
            array('CURLFTP_CREATE_DIR', '7.0.6', array(532), '7.1', '7.0'),
            array('CURLFTP_CREATE_DIR_NONE', '7.0.6', array(533), '7.1', '7.0'),
            array('CURLFTP_CREATE_DIR_RETRY', '7.0.6', array(534), '7.1', '7.0'),
            array('CURLHEADER_SEPARATE', '7.0.6', array(535), '7.1', '7.0'),
            array('CURLHEADER_UNIFIED', '7.0.6', array(536), '7.1', '7.0'),
            array('CURLMOPT_CHUNK_LENGTH_PENALTY_SIZE', '7.0.6', array(537), '7.1', '7.0'),
            array('CURLMOPT_CONTENT_LENGTH_PENALTY_SIZE', '7.0.6', array(538), '7.1', '7.0'),
            array('CURLMOPT_MAX_HOST_CONNECTIONS', '7.0.6', array(539), '7.1', '7.0'),
            array('CURLMOPT_MAX_PIPELINE_LENGTH', '7.0.6', array(540), '7.1', '7.0'),
            array('CURLMOPT_MAX_TOTAL_CONNECTIONS', '7.0.6', array(541), '7.1', '7.0'),
            array('CURLOPT_CONNECT_TO', '7.0.6', array(542), '7.1', '7.0'),
            array('CURLOPT_DEFAULT_PROTOCOL', '7.0.6', array(543), '7.1', '7.0'),
            array('CURLOPT_DNS_INTERFACE', '7.0.6', array(544), '7.1', '7.0'),
            array('CURLOPT_DNS_LOCAL_IP4', '7.0.6', array(545), '7.1', '7.0'),
            array('CURLOPT_DNS_LOCAL_IP6', '7.0.6', array(546), '7.1', '7.0'),
            array('CURLOPT_EXPECT_100_TIMEOUT_MS', '7.0.6', array(547), '7.1', '7.0'),
            array('CURLOPT_HEADEROPT', '7.0.6', array(548), '7.1', '7.0'),
            array('CURLOPT_LOGIN_OPTIONS', '7.0.6', array(549), '7.1', '7.0'),
            array('CURLOPT_PATH_AS_IS', '7.0.6', array(550), '7.1', '7.0'),
            array('CURLOPT_PINNEDPUBLICKEY', '7.0.6', array(551), '7.1', '7.0'),
            array('CURLOPT_PIPEWAIT', '7.0.6', array(552), '7.1', '7.0'),
            array('CURLOPT_PROXY_SERVICE_NAME', '7.0.6', array(553), '7.1', '7.0'),
            array('CURLOPT_PROXYHEADER', '7.0.6', array(554), '7.1', '7.0'),
            array('CURLOPT_SASL_IR', '7.0.6', array(555), '7.1', '7.0'),
            array('CURLOPT_SERVICE_NAME', '7.0.6', array(556), '7.1', '7.0'),
            array('CURLOPT_SSL_ENABLE_ALPN', '7.0.6', array(557), '7.1', '7.0'),
            array('CURLOPT_SSL_ENABLE_NPN', '7.0.6', array(558), '7.1', '7.0'),
            array('CURLOPT_SSL_FALSESTART', '7.0.6', array(559), '7.1', '7.0'),
            array('CURLOPT_SSL_VERIFYSTATUS', '7.0.6', array(560), '7.1', '7.0'),
            array('CURLOPT_STREAM_WEIGHT', '7.0.6', array(561), '7.1', '7.0'),
            array('CURLOPT_TCP_FASTOPEN', '7.0.6', array(562), '7.1', '7.0'),
            array('CURLOPT_TFTP_NO_OPTIONS', '7.0.6', array(563), '7.1', '7.0'),
            array('CURLOPT_UNIX_SOCKET_PATH', '7.0.6', array(564), '7.1', '7.0'),
            array('CURLOPT_XOAUTH2_BEARER', '7.0.6', array(565), '7.1', '7.0'),
            array('CURLPROTO_SMB', '7.0.6', array(566), '7.1', '7.0'),
            array('CURLPROTO_SMBS', '7.0.6', array(567), '7.1', '7.0'),
            array('CURLPROXY_HTTP_1_0', '7.0.6', array(568), '7.1', '7.0'),
            array('CURLSSH_AUTH_AGENT', '7.0.6', array(569), '7.1', '7.0'),
            array('CURLSSLOPT_NO_REVOKE', '7.0.6', array(570), '7.1', '7.0'),
            array('PHP_FD_SETSIZE', '7.0', array(571), '7.1'),
            array('CURLMOPT_PUSHFUNCTION', '7.0', array(572), '7.1'),
            array('CURL_PUSH_OK', '7.0', array(573), '7.1'),
            array('CURL_PUSH_DENY', '7.0', array(574), '7.1'),
            array('FILTER_FLAG_EMAIL_UNICODE', '7.0', array(575), '7.1'),
            array('IMAGETYPE_WEBP', '7.0', array(576), '7.1'),
            array('JSON_UNESCAPED_LINE_TERMINATORS', '7.0', array(577), '7.1'),
            array('LDAP_OPT_X_SASL_NOCANON', '7.0', array(578), '7.1'),
            array('LDAP_OPT_X_SASL_USERNAME', '7.0', array(579), '7.1'),
            array('LDAP_OPT_X_TLS_CACERTDIR', '7.0', array(580), '7.1'),
            array('LDAP_OPT_X_TLS_CACERTFILE', '7.0', array(581), '7.1'),
            array('LDAP_OPT_X_TLS_CERTFILE', '7.0', array(582), '7.1'),
            array('LDAP_OPT_X_TLS_CIPHER_SUITE', '7.0', array(583), '7.1'),
            array('LDAP_OPT_X_TLS_KEYFILE', '7.0', array(584), '7.1'),
            array('LDAP_OPT_X_TLS_RANDOM_FILE', '7.0', array(585), '7.1'),
            array('LDAP_OPT_X_TLS_CRLCHECK', '7.0', array(586), '7.1'),
            array('LDAP_OPT_X_TLS_CRL_NONE', '7.0', array(587), '7.1'),
            array('LDAP_OPT_X_TLS_CRL_PEER', '7.0', array(588), '7.1'),
            array('LDAP_OPT_X_TLS_CRL_ALL', '7.0', array(589), '7.1'),
            array('LDAP_OPT_X_TLS_DHFILE', '7.0', array(590), '7.1'),
            array('LDAP_OPT_X_TLS_CRLFILE', '7.0', array(591), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_MIN', '7.0', array(592), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_SSL2', '7.0', array(593), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_SSL3', '7.0', array(594), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_TLS1_0', '7.0', array(595), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_TLS1_1', '7.0', array(596), '7.1'),
            array('LDAP_OPT_X_TLS_PROTOCOL_TLS1_2', '7.0', array(597), '7.1'),
            array('LDAP_OPT_X_TLS_PACKAGE', '7.0', array(598), '7.1'),
            array('LDAP_OPT_X_KEEPALIVE_IDLE', '7.0', array(599), '7.1'),
            array('LDAP_OPT_X_KEEPALIVE_PROBES', '7.0', array(600), '7.1'),
            array('LDAP_OPT_X_KEEPALIVE_INTERVAL', '7.0', array(601), '7.1'),
            array('PGSQL_NOTICE_LAST', '7.0', array(602), '7.1'),
            array('PGSQL_NOTICE_ALL', '7.0', array(603), '7.1'),
            array('PGSQL_NOTICE_CLEAR', '7.0', array(604), '7.1'),
            array('MT_RAND_PHP', '7.0', array(605), '7.1'),
            array('SQLITE3_DETERMINISTIC', '7.1.3', array(606), '7.2', '7.1'),
            array('PHP_OS_FAMILY', '7.1', array(607), '7.2'),
            array('PHP_FLOAT_DIG', '7.1', array(608), '7.2'),
            array('PHP_FLOAT_EPSILON', '7.1', array(609), '7.2'),
            array('PHP_FLOAT_MIN', '7.1', array(610), '7.2'),
            array('PHP_FLOAT_MAX', '7.1', array(611), '7.2'),
            array('PASSWORD_ARGON2I', '7.1', array(612), '7.2'),
            array('PASSWORD_ARGON2_DEFAULT_MEMORY_COST', '7.1', array(613), '7.2'),
            array('PASSWORD_ARGON2_DEFAULT_TIME_COST', '7.1', array(614), '7.2'),
            array('PASSWORD_ARGON2_DEFAULT_THREADS', '7.1', array(615), '7.2'),
            array('FILEINFO_EXTENSION', '7.1', array(616), '7.2'),
            array('IMG_EFFECT_MULTIPLY', '7.1', array(617), '7.2'),
            array('IMG_BMP', '7.1', array(618), '7.2'),
            array('JSON_INVALID_UTF8_IGNORE', '7.1', array(619), '7.2'),
            array('JSON_INVALID_UTF8_SUBSTITUTE', '7.1', array(620), '7.2'),
            array('LDAP_EXOP_START_TLS', '7.1', array(621), '7.2'),
            array('LDAP_EXOP_MODIFY_PASSWD', '7.1', array(622), '7.2'),
            array('LDAP_EXOP_REFRESH', '7.1', array(623), '7.2'),
            array('LDAP_EXOP_WHO_AM_I', '7.1', array(624), '7.2'),
            array('LDAP_EXOP_TURN', '7.1', array(625), '7.2'),
            array('PREG_UNMATCHED_AS_NULL', '7.1', array(626), '7.2'),
            array('SODIUM_LIBRARY_VERSION', '7.1', array(627), '7.2'),
            array('SODIUM_LIBRARY_MAJOR_VERSION', '7.1', array(628), '7.2'),
            array('SODIUM_LIBRARY_MINOR_VERSION', '7.1', array(629), '7.2'),
            array('SODIUM_BASE64_VARIANT_ORIGINAL', '7.1', array(866), '7.2'),
            array('SODIUM_BASE64_VARIANT_ORIGINAL_NO_PADDING', '7.1', array(867), '7.2'),
            array('SODIUM_BASE64_VARIANT_URLSAFE', '7.1', array(868), '7.2'),
            array('SODIUM_BASE64_VARIANT_URLSAFE_NO_PADDING', '7.1', array(869), '7.2'),
            array('SODIUM_CRYPTO_AEAD_AES256GCM_KEYBYTES', '7.1', array(630), '7.2'),
            array('SODIUM_CRYPTO_AEAD_AES256GCM_NSECBYTES', '7.1', array(631), '7.2'),
            array('SODIUM_CRYPTO_AEAD_AES256GCM_NPUBBYTES', '7.1', array(632), '7.2'),
            array('SODIUM_CRYPTO_AEAD_AES256GCM_ABYTES', '7.1', array(633), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_KEYBYTES', '7.1', array(634), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_NSECBYTES', '7.1', array(635), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_NPUBBYTES', '7.1', array(636), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_ABYTES', '7.1', array(637), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_IETF_KEYBYTES', '7.1', array(638), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_IETF_NSECBYTES', '7.1', array(639), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_IETF_NPUBBYTES', '7.1', array(640), '7.2'),
            array('SODIUM_CRYPTO_AEAD_CHACHA20POLY1305_IETF_ABYTES', '7.1', array(641), '7.2'),
            array('SODIUM_CRYPTO_AEAD_XCHACHA20POLY1305_IETF_KEYBYTES', '7.1', array(870), '7.2'),
            array('SODIUM_CRYPTO_AEAD_XCHACHA20POLY1305_IETF_NSECBYTES', '7.1', array(871), '7.2'),
            array('SODIUM_CRYPTO_AEAD_XCHACHA20POLY1305_IETF_NPUBBYTES', '7.1', array(872), '7.2'),
            array('SODIUM_CRYPTO_AEAD_XCHACHA20POLY1305_IETF_ABYTES', '7.1', array(873), '7.2'),
            array('SODIUM_CRYPTO_AUTH_BYTES', '7.1', array(642), '7.2'),
            array('SODIUM_CRYPTO_AUTH_KEYBYTES', '7.1', array(643), '7.2'),
            array('SODIUM_CRYPTO_BOX_SEALBYTES', '7.1', array(644), '7.2'),
            array('SODIUM_CRYPTO_BOX_SECRETKEYBYTES', '7.1', array(645), '7.2'),
            array('SODIUM_CRYPTO_BOX_PUBLICKEYBYTES', '7.1', array(646), '7.2'),
            array('SODIUM_CRYPTO_BOX_KEYPAIRBYTES', '7.1', array(647), '7.2'),
            array('SODIUM_CRYPTO_BOX_MACBYTES', '7.1', array(648), '7.2'),
            array('SODIUM_CRYPTO_BOX_NONCEBYTES', '7.1', array(649), '7.2'),
            array('SODIUM_CRYPTO_BOX_SEEDBYTES', '7.1', array(650), '7.2'),
            array('SODIUM_CRYPTO_KDF_BYTES_MIN', '7.1', array(651), '7.2'),
            array('SODIUM_CRYPTO_KDF_BYTES_MAX', '7.1', array(652), '7.2'),
            array('SODIUM_CRYPTO_KDF_CONTEXTBYTES', '7.1', array(653), '7.2'),
            array('SODIUM_CRYPTO_KDF_KEYBYTES', '7.1', array(654), '7.2'),
            array('SODIUM_CRYPTO_KX_SEEDBYTES', '7.1', array(655), '7.2'),
            array('SODIUM_CRYPTO_KX_SESSIONKEYBYTES', '7.1', array(656), '7.2'),
            array('SODIUM_CRYPTO_KX_PUBLICKEYBYTES', '7.1', array(657), '7.2'),
            array('SODIUM_CRYPTO_KX_SECRETKEYBYTES', '7.1', array(658), '7.2'),
            array('SODIUM_CRYPTO_KX_KEYPAIRBYTES', '7.1', array(659), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_BYTES', '7.1', array(660), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_BYTES_MIN', '7.1', array(661), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_BYTES_MAX', '7.1', array(662), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_KEYBYTES', '7.1', array(663), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_KEYBYTES_MIN', '7.1', array(664), '7.2'),
            array('SODIUM_CRYPTO_GENERICHASH_KEYBYTES_MAX', '7.1', array(665), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_ALG_ARGON2I13', '7.1', array(666), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_ALG_ARGON2ID13', '7.1', array(865), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_ALG_DEFAULT', '7.1', array(667), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SALTBYTES', '7.1', array(668), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_STRPREFIX', '7.1', array(669), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_OPSLIMIT_INTERACTIVE', '7.1', array(670), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_MEMLIMIT_INTERACTIVE', '7.1', array(671), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_OPSLIMIT_MODERATE', '7.1', array(672), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_MEMLIMIT_MODERATE', '7.1', array(673), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_OPSLIMIT_SENSITIVE', '7.1', array(674), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_MEMLIMIT_SENSITIVE', '7.1', array(675), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_SALTBYTES', '7.1', array(676), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_STRPREFIX', '7.1', array(677), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_OPSLIMIT_INTERACTIVE', '7.1', array(678), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_MEMLIMIT_INTERACTIVE', '7.1', array(679), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_OPSLIMIT_SENSITIVE', '7.1', array(680), '7.2'),
            array('SODIUM_CRYPTO_PWHASH_SCRYPTSALSA208SHA256_MEMLIMIT_SENSITIVE', '7.1', array(681), '7.2'),
            array('SODIUM_CRYPTO_SCALARMULT_BYTES', '7.1', array(682), '7.2'),
            array('SODIUM_CRYPTO_SCALARMULT_SCALARBYTES', '7.1', array(683), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_ABYTES', '7.1', array(874), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_HEADERBYTES', '7.1', array(875), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_KEYBYTES', '7.1', array(876), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_MESSAGEBYTES_MAX', '7.1', array(877), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_TAG_MESSAGE', '7.1', array(878), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_TAG_PUSH', '7.1', array(879), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_TAG_REKEY', '7.1', array(880), '7.2'),
            array('SODIUM_CRYPTO_SECRETSTREAM_XCHACHA20POLY1305_TAG_FINAL', '7.1', array(881), '7.2'),
            array('SODIUM_CRYPTO_SHORTHASH_BYTES', '7.1', array(684), '7.2'),
            array('SODIUM_CRYPTO_SHORTHASH_KEYBYTES', '7.1', array(685), '7.2'),
            array('SODIUM_CRYPTO_SECRETBOX_KEYBYTES', '7.1', array(686), '7.2'),
            array('SODIUM_CRYPTO_SECRETBOX_MACBYTES', '7.1', array(687), '7.2'),
            array('SODIUM_CRYPTO_SECRETBOX_NONCEBYTES', '7.1', array(688), '7.2'),
            array('SODIUM_CRYPTO_SIGN_BYTES', '7.1', array(689), '7.2'),
            array('SODIUM_CRYPTO_SIGN_SEEDBYTES', '7.1', array(690), '7.2'),
            array('SODIUM_CRYPTO_SIGN_PUBLICKEYBYTES', '7.1', array(691), '7.2'),
            array('SODIUM_CRYPTO_SIGN_SECRETKEYBYTES', '7.1', array(692), '7.2'),
            array('SODIUM_CRYPTO_SIGN_KEYPAIRBYTES', '7.1', array(693), '7.2'),
            array('SODIUM_CRYPTO_STREAM_NONCEBYTES', '7.1', array(694), '7.2'),
            array('SODIUM_CRYPTO_STREAM_KEYBYTES', '7.1', array(695), '7.2'),

            array('CURLOPT_REQUEST_TARGET', '7.2', array(699), '7.3'),
            array('FILTER_SANITIZE_ADD_SLASHES', '7.2', array(700), '7.3'),
            array('JSON_THROW_ON_ERROR', '7.2', array(701), '7.3'),
            array('MB_CASE_FOLD', '7.2', array(702), '7.3'),
            array('MB_CASE_UPPER_SIMPLE', '7.2', array(703), '7.3'),
            array('MB_CASE_LOWER_SIMPLE', '7.2', array(704), '7.3'),
            array('MB_CASE_TITLE_SIMPLE', '7.2', array(705), '7.3'),
            array('MB_CASE_FOLD_SIMPLE', '7.2', array(706), '7.3'),
            array('PGSQL_DIAG_SCHEMA_NAME', '7.2', array(707), '7.3'),
            array('PGSQL_DIAG_TABLE_NAME', '7.2', array(708), '7.3'),
            array('PGSQL_DIAG_COLUMN_NAME', '7.2', array(709), '7.3'),
            array('PGSQL_DIAG_DATATYPE_NAME', '7.2', array(710), '7.3'),
            array('PGSQL_DIAG_CONSTRAINT_NAME', '7.2', array(711), '7.3'),
            array('PGSQL_DIAG_SEVERITY_NONLOCALIZED', '7.2', array(712), '7.3'),
            array('PASSWORD_ARGON2ID', '7.2', array(713), '7.3'),
            array('CURLAUTH_GSSAPI', '7.2', array(714), '7.3'),
            array('CURLE_WEIRD_SERVER_REPLY', '7.2', array(715), '7.3'),
            array('CURLINFO_CONTENT_LENGTH_DOWNLOAD_T', '7.2', array(716), '7.3'),
            array('CURLINFO_CONTENT_LENGTH_UPLOAD_T', '7.2', array(717), '7.3'),
            array('CURLINFO_HTTP_VERSION', '7.2', array(718), '7.3'),
            array('CURLINFO_PROTOCOL', '7.2', array(719), '7.3'),
            array('CURLINFO_PROXY_SSL_VERIFYRESULT', '7.2', array(720), '7.3'),
            array('CURLINFO_SCHEME', '7.2', array(721), '7.3'),
            array('CURLINFO_SIZE_DOWNLOAD_T', '7.2', array(722), '7.3'),
            array('CURLINFO_SIZE_UPLOAD_T', '7.2', array(723), '7.3'),
            array('CURLINFO_SPEED_DOWNLOAD_T', '7.2', array(724), '7.3'),
            array('CURLINFO_SPEED_UPLOAD_T', '7.2', array(725), '7.3'),
            array('CURL_MAX_READ_SIZE', '7.2', array(726), '7.3'),
            array('CURLOPT_ABSTRACT_UNIX_SOCKET', '7.2', array(727), '7.3'),
            array('CURLOPT_KEEP_SENDING_ON_ERROR', '7.2', array(728), '7.3'),
            array('CURLOPT_PRE_PROXY', '7.2', array(729), '7.3'),
            array('CURLOPT_PROXY_CAINFO', '7.2', array(730), '7.3'),
            array('CURLOPT_PROXY_CAPATH', '7.2', array(731), '7.3'),
            array('CURLOPT_PROXY_CRLFILE', '7.2', array(732), '7.3'),
            array('CURLOPT_PROXY_KEYPASSWD', '7.2', array(733), '7.3'),
            array('CURLOPT_PROXY_PINNEDPUBLICKEY', '7.2', array(734), '7.3'),
            array('CURLOPT_PROXY_SSLCERT', '7.2', array(735), '7.3'),
            array('CURLOPT_PROXY_SSLCERTTYPE', '7.2', array(736), '7.3'),
            array('CURLOPT_PROXY_SSL_CIPHER_LIST', '7.2', array(737), '7.3'),
            array('CURLOPT_PROXY_SSLKEY', '7.2', array(738), '7.3'),
            array('CURLOPT_PROXY_SSLKEYTYPE', '7.2', array(739), '7.3'),
            array('CURLOPT_PROXY_SSL_OPTIONS', '7.2', array(740), '7.3'),
            array('CURLOPT_PROXY_SSL_VERIFYHOST', '7.2', array(741), '7.3'),
            array('CURLOPT_PROXY_SSL_VERIFYPEER', '7.2', array(742), '7.3'),
            array('CURLOPT_PROXY_SSLVERSION', '7.2', array(743), '7.3'),
            array('CURLOPT_PROXY_TLSAUTH_PASSWORD', '7.2', array(744), '7.3'),
            array('CURLOPT_PROXY_TLSAUTH_TYPE', '7.2', array(745), '7.3'),
            array('CURLOPT_PROXY_TLSAUTH_USERNAME', '7.2', array(746), '7.3'),
            array('CURLOPT_SOCKS5_AUTH', '7.2', array(747), '7.3'),
            array('CURLOPT_SUPPRESS_CONNECT_HEADERS', '7.2', array(748), '7.3'),
            array('CURLPROXY_HTTPS', '7.2', array(749), '7.3'),
            array('CURL_SSLVERSION_MAX_DEFAULT', '7.2', array(750), '7.3'),
            array('CURL_SSLVERSION_MAX_NONE', '7.2', array(751), '7.3'),
            array('CURL_SSLVERSION_MAX_TLSv1_0', '7.2', array(752), '7.3'),
            array('CURL_SSLVERSION_MAX_TLSv1_1', '7.2', array(753), '7.3'),
            array('CURL_SSLVERSION_MAX_TLSv1_2', '7.2', array(754), '7.3'),
            array('CURL_SSLVERSION_MAX_TLSv1_3', '7.2', array(755), '7.3'),
            array('CURL_SSLVERSION_TLSv1_3', '7.2', array(756), '7.3'),
            array('CURL_VERSION_HTTPS_PROXY', '7.2', array(757), '7.3'),
            array('CURLAUTH_BEARER', '7.2', array(758), '7.3'),
            array('CURLINFO_APPCONNECT_TIME_T', '7.2', array(759), '7.3'),
            array('CURLINFO_CONNECT_TIME_T', '7.2', array(760), '7.3'),
            array('CURLINFO_FILETIME_T', '7.2', array(761), '7.3'),
            array('CURLINFO_NAMELOOKUP_TIME_T', '7.2', array(762), '7.3'),
            array('CURLINFO_PRETRANSFER_TIME_T', '7.2', array(763), '7.3'),
            array('CURLINFO_REDIRECT_TIME_T', '7.2', array(764), '7.3'),
            array('CURLINFO_STARTTRANSFER_TIME_T', '7.2', array(765), '7.3'),
            array('CURLINFO_TOTAL_TIME_T', '7.2', array(766), '7.3'),
            array('CURL_LOCK_DATA_CONNECT', '7.2', array(767), '7.3'),
            array('CURL_LOCK_DATA_PSL', '7.2', array(768), '7.3'),
            array('CURLOPT_DISALLOW_USERNAME_IN_URL', '7.2', array(769), '7.3'),
            array('CURLOPT_DNS_SHUFFLE_ADDRESSES', '7.2', array(770), '7.3'),
            array('CURLOPT_HAPPY_EYEBALLS_TIMEOUT_MS', '7.2', array(771), '7.3'),
            array('CURLOPT_HAPROXYPROTOCOL', '7.2', array(772), '7.3'),
            array('CURLOPT_PROXY_TLS13_CIPHERS', '7.2', array(773), '7.3'),
            array('CURLOPT_SSH_COMPRESSION', '7.2', array(774), '7.3'),
            array('CURLOPT_TIMEVALUE_LARGE', '7.2', array(775), '7.3'),
            array('CURLOPT_TLS13_CIPHERS', '7.2', array(776), '7.3'),
            array('CURLSSH_AUTH_GSSAPI', '7.2', array(777), '7.3'),
            array('CURL_VERSION_ASYNCHDNS', '7.2', array(778), '7.3'),
            array('CURL_VERSION_BROTLI', '7.2', array(779), '7.3'),
            array('CURL_VERSION_CONV', '7.2', array(780), '7.3'),
            array('CURL_VERSION_DEBUG', '7.2', array(781), '7.3'),
            array('CURL_VERSION_GSSAPI', '7.2', array(782), '7.3'),
            array('CURL_VERSION_GSSNEGOTIATE', '7.2', array(783), '7.3'),
            array('CURL_VERSION_IDN', '7.2', array(784), '7.3'),
            array('CURL_VERSION_LARGEFILE', '7.2', array(785), '7.3'),
            array('CURL_VERSION_MULTI_SSL', '7.2', array(786), '7.3'),
            array('CURL_VERSION_NTLM', '7.2', array(787), '7.3'),
            array('CURL_VERSION_NTLM_WB', '7.2', array(788), '7.3'),
            array('CURL_VERSION_SPNEGO', '7.2', array(789), '7.3'),
            array('CURL_VERSION_SSPI', '7.2', array(790), '7.3'),
            array('CURL_VERSION_TLSAUTH_SRP', '7.2', array(791), '7.3'),
            array('STREAM_CRYPTO_PROTO_SSLv3', '7.2', array(792), '7.3'),
            array('STREAM_CRYPTO_PROTO_TLSv1_0', '7.2', array(793), '7.3'),
            array('STREAM_CRYPTO_PROTO_TLSv1_1', '7.2', array(794), '7.3'),
            array('STREAM_CRYPTO_PROTO_TLSv1_2', '7.2', array(795), '7.3'),
            array('LDAP_CONTROL_MANAGEDSAIT', '7.2', array(796), '7.3'),
            array('LDAP_CONTROL_PROXY_AUTHZ', '7.2', array(797), '7.3'),
            array('LDAP_CONTROL_SUBENTRIES', '7.2', array(798), '7.3'),
            array('LDAP_CONTROL_VALUESRETURNFILTER', '7.2', array(799), '7.3'),
            array('LDAP_CONTROL_ASSERT', '7.2', array(800), '7.3'),
            array('LDAP_CONTROL_PRE_READ', '7.2', array(801), '7.3'),
            array('LDAP_CONTROL_POST_READ', '7.2', array(802), '7.3'),
            array('LDAP_CONTROL_SORTREQUEST', '7.2', array(803), '7.3'),
            array('LDAP_CONTROL_SORTRESPONSE', '7.2', array(804), '7.3'),
            array('LDAP_CONTROL_PAGEDRESULTS', '7.2', array(805), '7.3'),
            array('LDAP_CONTROL_AUTHZID_REQUEST', '7.2', array(806), '7.3'),
            array('LDAP_CONTROL_AUTHZID_RESPONSE', '7.2', array(807), '7.3'),
            array('LDAP_CONTROL_SYNC', '7.2', array(808), '7.3'),
            array('LDAP_CONTROL_SYNC_STATE', '7.2', array(809), '7.3'),
            array('LDAP_CONTROL_SYNC_DONE', '7.2', array(810), '7.3'),
            array('LDAP_CONTROL_DONTUSECOPY', '7.2', array(811), '7.3'),
            array('LDAP_CONTROL_PASSWORDPOLICYREQUEST', '7.2', array(812), '7.3'),
            array('LDAP_CONTROL_PASSWORDPOLICYRESPONSE', '7.2', array(813), '7.3'),
            array('LDAP_CONTROL_X_INCREMENTAL_VALUES', '7.2', array(814), '7.3'),
            array('LDAP_CONTROL_X_DOMAIN_SCOPE', '7.2', array(815), '7.3'),
            array('LDAP_CONTROL_X_PERMISSIVE_MODIFY', '7.2', array(816), '7.3'),
            array('LDAP_CONTROL_X_SEARCH_OPTIONS', '7.2', array(817), '7.3'),
            array('LDAP_CONTROL_X_TREE_DELETE', '7.2', array(818), '7.3'),
            array('LDAP_CONTROL_X_EXTENDED_DN', '7.2', array(819), '7.3'),
            array('LDAP_CONTROL_VLVREQUEST', '7.2', array(820), '7.3'),
            array('LDAP_CONTROL_VLVRESPONSE', '7.2', array(821), '7.3'),

            array('CURL_VERSION_ALTSVC', '7.3.5', array(860), '7.4', '7.3'),
            array('CURL_VERSION_CURLDEBUG', '7.3.5', array(861), '7.4', '7.3'),

            array('IMG_FILTER_SCATTER', '7.3', array(863), '7.4'),
            array('MB_ONIGURUMA_VERSION', '7.3', array(824), '7.4'),
            array('SO_LABEL', '7.3', array(825), '7.4'),
            array('SO_PEERLABEL', '7.3', array(826), '7.4'),
            array('SO_LISTENQLIMIT', '7.3', array(827), '7.4'),
            array('SO_LISTENQLEN', '7.3', array(828), '7.4'),
            array('SO_USER_COOKIE', '7.3', array(829), '7.4'),
            array('PASSWORD_ARGON2_PROVIDER', '7.3', array(864), '7.4'),
            array('PHP_WINDOWS_EVENT_CTRL_C', '7.3', array(830), '7.4'),
            array('PHP_WINDOWS_EVENT_CTRL_BREAK', '7.3', array(831), '7.4'),
            array('T_BAD_CHARACTER', '7.3', array(862), '7.4'),
            array('TIDY_TAG_ARTICLE', '7.3', array(832), '7.4'),
            array('TIDY_TAG_ASIDE', '7.3', array(833), '7.4'),
            array('TIDY_TAG_AUDIO', '7.3', array(834), '7.4'),
            array('TIDY_TAG_BDI', '7.3', array(835), '7.4'),
            array('TIDY_TAG_CANVAS', '7.3', array(836), '7.4'),
            array('TIDY_TAG_COMMAND', '7.3', array(837), '7.4'),
            array('TIDY_TAG_DATALIST', '7.3', array(838), '7.4'),
            array('TIDY_TAG_DETAILS', '7.3', array(839), '7.4'),
            array('TIDY_TAG_DIALOG', '7.3', array(840), '7.4'),
            array('TIDY_TAG_FIGCAPTION', '7.3', array(841), '7.4'),
            array('TIDY_TAG_FIGURE', '7.3', array(842), '7.4'),
            array('TIDY_TAG_FOOTER', '7.3', array(843), '7.4'),
            array('TIDY_TAG_HEADER', '7.3', array(844), '7.4'),
            array('TIDY_TAG_HGROUP', '7.3', array(845), '7.4'),
            array('TIDY_TAG_MAIN', '7.3', array(846), '7.4'),
            array('TIDY_TAG_MARK', '7.3', array(847), '7.4'),
            array('TIDY_TAG_MENUITEM', '7.3', array(848), '7.4'),
            array('TIDY_TAG_METER', '7.3', array(849), '7.4'),
            array('TIDY_TAG_NAV', '7.3', array(850), '7.4'),
            array('TIDY_TAG_OUTPUT', '7.3', array(851), '7.4'),
            array('TIDY_TAG_PROGRESS', '7.3', array(852), '7.4'),
            array('TIDY_TAG_SECTION', '7.3', array(853), '7.4'),
            array('TIDY_TAG_SOURCE', '7.3', array(854), '7.4'),
            array('TIDY_TAG_SUMMARY', '7.3', array(855), '7.4'),
            array('TIDY_TAG_TEMPLATE', '7.3', array(856), '7.4'),
            array('TIDY_TAG_TIME', '7.3', array(857), '7.4'),
            array('TIDY_TAG_TRACK', '7.3', array(858), '7.4'),
            array('TIDY_TAG_VIDEO', '7.3', array(859), '7.4'),
        );
    }


    /**
     * Test constants that shouldn't be flagged by this sniff.
     *
     * @dataProvider dataNoFalsePositives
     *
     * @param int $line The line number.
     *
     * @return void
     */
    public function testNoFalsePositives($line)
    {
        $file = $this->sniffFile(__FILE__, '4.4'); // Low version below the first addition.
        $this->assertNoViolation($file, $line);
    }

    /**
     * Data provider.
     *
     * @see testNoFalsePositives()
     *
     * @return array
     */
    public function dataNoFalsePositives()
    {
        return array(
            array(4),
            array(5),
        );
    }


    /**
     * Verify no notices are thrown at all.
     *
     * @return void
     */
    public function testNoViolationsInFileOnValidVersion()
    {
        $file = $this->sniffFile(__FILE__, '99.0'); // High version beyond newest addition.
        $this->assertNoViolation($file);
    }
}
